import sqlalchemy as sa
import pandas as pd


class DataAccessProvider:

    def __init__(self, dsn=None, area_gt=None, pixcount_gt=None, provincie=None, limit=None, gws_gewasc=None):

        self.engine = sa.create_engine(dsn)
        meta = sa.MetaData(self.engine)
        self.tbl_perc_info = sa.Table('perceels_info', meta, autoload=True)
        self.s2_observations = sa.Table("s2_observations", meta, autoload=True)
        if limit is None:
            limit = int(1e9)
        self.limit = limit

        self.perc_stmt = sa.select([self.tbl_perc_info])
        if area_gt is not None:
            self.perc_stmt.append_whereclause(self.tbl_perc_info.c.area_ha > area_gt)
        if gws_gewasc is not None:
            self.perc_stmt.append_whereclause(self.tbl_perc_info.c.gws_gewasc == gws_gewasc)
        if pixcount_gt is not None:
            self.perc_stmt.append_whereclause(self.tbl_perc_info.c.pixcount > pixcount_gt)
        if provincie is not None:
            self.perc_stmt.append_whereclause(self.tbl_perc_info.c.provincie == provincie)

        s = sa.select([sa.func.count()]).select_from(self.perc_stmt)
        self.count = s.execute().fetchone()[0]

    def __iter__(self):

        r = self.perc_stmt.limit(self.limit).execute()
        rows = r.fetchmany(100)
        while rows:
            for row in rows:
                s = sa.select([self.s2_observations],
                              sa.and_(self.s2_observations.c.fieldID==row.fieldID),
                              order_by={self.s2_observations.c.day})
                df = pd.read_sql(s, self.engine)
                df = df.drop(columns="fieldID")
                df.index = pd.to_datetime(df.day)
                yield row, df
            rows = r.fetchmany()


    def __len__(self):
        return self.count

