import sqlalchemy
from sqlalchemy import MetaData, Table, Column, Integer, Date, Float, Text, create_engine

def take_first(iter):
    for i in iter:
        return i


def count_lines(files):
    """Checks the number of lines in the input CSV files.

    They should all be the same else throw an error.
    """
    print("Checking file row counts...")
    counts = {}
    for band, fname in files.items():
        with open(fname) as my_file:
            c = sum(1 for _ in my_file)
        counts[fname] = c
        print(f"  - {fname}: {c}")

    if len(set(counts.values())) > 1:
        msg = "CSV files do not have the same number of rows!"
        raise RuntimeError(msg)

    return take_first(counts.values())


def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end = printEnd)
    # Print New Line on Complete
    if iteration == total:
        print()


def prepare_db(dsn, table_name, bands):
    engine = create_engine(dsn)
    meta = MetaData(engine)
    tbl = Table(table_name, meta,
                Column('fieldID', Integer, primary_key=True, nullable=False),
                Column('day', Date, primary_key=True, nullable=False),
                )
    for col_name in bands:
        tbl.append_column(Column(col_name, Float, nullable=True))

    try:
        tbl.drop()
    except sqlalchemy.exc.OperationalError:
        pass
    tbl.create()

    return engine