from pathlib import Path

import geopandas as gpd
import pandas as pd
import sqlalchemy as sa
import numpy as np


def main():
    fname_percelen = Path.cwd() / "BRP" / "gewaspercelen_2019.shp"
    # fname_percelen = Path.cwd() / "BRP" / "BRP_10rows.shp"
    df = gpd.read_file(fname_percelen)
    df["area_ha"]  = df.geometry.area/1e4
    df = df.set_index("fieldid")

    fname_counts = Path.cwd() / "Optisch" / "perceelscount.csv"
    df_counts = pd.read_csv(fname_counts)
    df_counts.set_index("field_ID", inplace=True)

    df["pixcount"] = df_counts.pixcount

    df_out = pd.DataFrame({"fieldID": df.index,
                           "year": df.year,
                           "pixcount": df.pixcount,
                           "area_ha": df.area_ha,
                           "cat_gewasc": df.cat_gewasc.apply(str),
                           "gws_gewasc": df.gws_gewasc.astype(np.int32),
                           "gws_gewas": df.gws_gewas.apply(str),
                           "provincie": df.provincie.apply(str),
                           "gemeente": df.gemeente.apply(str),
                           "regio": df.regio.apply(str),
                           "pc4": df.PC4.apply(str),
                           "woonplaats": df.woonplaats.apply(str),
                           "waterschap": df.waterschap.apply(str),
                           })

    fname_percelen_db = Path.cwd() / "sentinel2_observations_2019.db3"
    dsn = f"sqlite:///{fname_percelen_db}"
    engine = sa.create_engine(dsn)
    df_out.to_sql("perceels_info", engine, if_exists="append", index=False)


if __name__ == "__main__":
    main()
